/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

(function () {

    window.onload = function () {
        window.addEventListener('tizenhwkey', function (ev) {
            if (ev.keyName === 'back') {
                var page = document.getElementsByClassName('ui-page-active')[0],
                    pageid = page ? page.id : '';
                if (pageid === 'main') {
                    try {
                        tizen.application.getCurrentApplication().exit();
                    } catch (ignore) {
                    }
                } else {
                    tau.back();
                }
            }
        });
    };


    /**
     * Contains the UI methods
     */
    var ui = {

        /**
         * Returns a list of attributes as jQuery items
         * @private
         * @param {string} attributes - List of attributes
         * @return {jQuery[]} List of jQuery items representing the attributes
         */
        _buildAttributeList: function (attributes) {
            var elements = [];

            // Creates a list of list item elements with two paragraphs embedded
            for (var i = 0; i < attributes.length; ++i) {
                var li = $('<li>').addClass('li-has-multiline')
                                  .text(attributes[i].title);

                $('<span>').addClass('ui-li-sub-text li-text-sub')
                           .text(attributes[i].value)
                           .appendTo(li);

                elements.push(li);
            }

            return elements;
        },


        /**
         * Initialises the UI by attaching various event handlers
         */
        init: function () {
            /**
             * Registers a listener for the main page
             */
            $('#main').one('pagebeforecreate', function () {
                // Maps DOM element IDs on click handlers
                var handlerMap = {
                    'btn-getWidget': widgetService.getWidget,
                    'btn-getPrimaryWidgetId': widgetService.getPrimaryWidgetId,
                    'btn-getSize': widgetService.getSize,
                    'btn-getName': widgetService.getName,
                    'btn-getVariant': widgetService.getVariant,
                    'btn-sendContent': widgetService.sendContent
                };

                // Attaches handlers to the list elements which test WidgetService methods
                $('.ui-listview li a').each(function () {
                    var element = $(this);
                    if (handlerMap[element.attr('id')]) {
                        element.on('click', handlerMap[element.attr('id')]);
                    }
                });

                $('.ui-popup button').on('click', function () {
                    tau.closePopup();
                });
            });

            /**
             * Registers a listener for the main page
             */
            $('#main').on('pagebeforeshow', function () {
                var listContainer = $(this).find('.ui-scroller')[0];

                // Registers a rotary event
                var SCROLL_STEP = 40;
                $(document).on('rotarydetent', function (e) {
                    if (e.detail.direction === 'CW') {
                        listContainer.scrollTop += SCROLL_STEP;
                    } else if (e.detail.direction === 'CCW') {
                        listContainer.scrollTop -= SCROLL_STEP;
                    }
                });
            });

            /**
             * Registers a listener for the result page
             */
            $('#result').on('pagebeforeshow', function () {
                var listContainer = $(this).find('.ui-scroller')[0];

                // Registers a rotary event
                var SCROLL_STEP = 40;
                $(document).on('rotarydetent', function (e) {
                    if (e.detail.direction === 'CW') {
                        listContainer.scrollTop += SCROLL_STEP;
                    } else if (e.detail.direction === 'CCW') {
                        listContainer.scrollTop -= SCROLL_STEP;
                    }
                });
            });

            /**
             * Registers listeners for popups
             */
            $('.ui-popup').on('popupbeforeshow', function () {
                var listContainer = $(this).find('.ui-popup-wrapper')[0];

                // Registers a rotary event
                var SCROLL_STEP = 40;
                $(document).on('rotarydetent', function (e) {
                    if (e.detail.direction === 'CW') {
                        listContainer.scrollTop += SCROLL_STEP;
                    } else if (e.detail.direction === 'CCW') {
                        listContainer.scrollTop -= SCROLL_STEP;
                    }
                });
            });
        },


        /**
         * Opens a page containing a list of attributes
         * @private
         * @param {string} attributes - List of attributes
         */
        openList: function (attributes) {
            var content = $('#result .ui-listview'),
                listElements = ui._buildAttributeList(attributes);

            content
                .empty()
                .append(listElements);

            tau.changePage('result');
        },


        /**
         * Opens a popup presenting the showContent method
         * @private
         * @param {function} finalise - Function which takes content data as an argument and is called on button click
         */
        openSendContent: function (finalise) {
            function onClick() {
                // Constructs the data to sent with input value
                var data = {
                    string: $('#content-input').val()
                };

                finalise(data);
            }

            $('#show-content button').one('click', onClick);
            tau.changePage('show-content');
        },


        /**
         * Opens a popup with an error message
         * @private
         * @param {string} message - Error message to display in the popup
         * @param {string} name - Error name to display in the popup
         */
        openErrorPopup: function (message, name) {
            if (!message) {
                return;
            }

            // Populates the popup with the message and name if exists
            $('#popup-error .ui-popup-content').text((name ? name + ': ' : '') + message);
            tau.openPopup('popup-error');
        }
    };


    /**
     * Contains the WidgetService logic
     */
    var widgetService = {

        /**
         * Unique ID of the widget
         * @private
         */
        _widgetId: null,


        /**
         * Width and height of the widget area
         * @private
         */
        _widgetSize: '2x2',


        /**
         * Initialises the WidgetService logic
         */
        init: function () {
            var packageId = tizen.application.getCurrentApplication().appInfo.packageId;
            widgetService._widgetId = tizen.widgetservice.getPrimaryWidgetId(packageId);
        },


        /**
         * Presents results returned by the tizen.widgetservice.getWidget() method
         * @private
         */
        getWidget: function () {
            try {
                var widget = tizen.widgetservice.getWidget(widgetService._widgetId);

                ui.openList([
                    { title: 'Widget ID', value: widget.id },
                    { title: 'Application ID', value: widget.applicationId },
                    { title: 'Package ID', value: widget.packageId },
                    { title: 'No display', value: widget.noDisplay }
                ]);
            } catch (err) {
                ui.openErrorPopup(err.message, err.name);
            }
        },


        /**
         * Presents results returned by the tizen.widgetservice.getPrimaryWidgetId() method
         * @private
         */
        getPrimaryWidgetId: function () {
            try {
                var primaryWidgetId = tizen.widgetservice.getPrimaryWidgetId(widgetService._widgetId);

                ui.openList([
                    { title: 'Primary widget ID', value: primaryWidgetId }
                ]);
            } catch (err) {
                ui.openErrorPopup(err.message, err.name);
            }
        },


        /**
         * Presents results returned by the tizen.widgetservice.getSize() method
         * @private
         */
        getSize: function () {
            try {
                var widgetDimension = tizen.widgetservice.getSize(widgetService._widgetSize);

                ui.openList([
                    { title: 'Widget size', value: widgetService._widgetSize },
                    { title: 'Width', value: widgetDimension.width },
                    { title: 'Height', value: widgetDimension.height }
                ]);
            } catch (err) {
                ui.openErrorPopup(err.message, err.name);
            }
        },


        /**
         * Presents results returned by the tizen.widgetservice.getName() method
         * @private
         */
        getName: function () {
            try {
                var widget = tizen.widgetservice.getWidget(widgetService._widgetId),
                    widgetName = widget.getName();

                ui.openList([
                    { title: 'Widget name', value: widgetName }
                ]);
            } catch (err) {
                ui.openErrorPopup(err.message, err.name);
            }
        },


        /**
         * Presents results returned by the tizen.widgetservice.getVariant() method
         * @private
         */
        getVariant: function () {
            try {
                var widget = tizen.widgetservice.getWidget(widgetService._widgetId),
                    variant = widget.getVariant(widgetService._widgetSize);

                ui.openList([
                    { title: 'Size type', value: variant.sizeType },
                    { title: 'Preview image path', value: variant.previewImagePath }
                ]);
            } catch (err) {
                ui.openErrorPopup(err.message, err.name);
            }
        },


        /**
         * Presents results returned by the tizen.widgetservice.sendContent() method
         * @private
         */
        sendContent: function () {
            try {
                var widget = tizen.widgetservice.getWidget(widgetService._widgetId);

                widget.getInstances(function (instances) {
                    function finalise(data) {
                        // Sends the data to each instance
                        for (var i = 0; i < instances.length; ++i) {
                            instances[i].sendContent(data, true);
                        }
                    }

                    ui.openSendContent(finalise);
                }, function (err) {
                    ui.openErrorPopup(err.message + ' (It seems that the widget has not been added)', err.name);
                });
            } catch (err) {
                ui.openErrorPopup(err.message, err.name);
            }
        }
    };


    widgetService.init();
    ui.init();
} ());